/*
   dtp.h

   Data Transfer Process

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (shesmax@ru.hilti.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: dtp.h,v 1.6 2000/08/15 17:31:54 shesmax Exp $ */

#ifndef __GUARD_DTP_H
#define __GUARD_DTP_H

/*
   Initialize the DTP. Returns RC_TRUE if case of successfull
   initialization, otherwise returns RC_FALSE
*/
int  InitDTP();
/* Closes data connection (if any) */
void ShutdownDTP();

/*
*   FTP command handlers
*/

int Appe( char* args );
int List( char* args );
int Mode( char* args );
int Nlst( char* args );
int Pasv( char* args );
int Port( char* args );
int Rest( char* args );
int Retr( char* args );
int Stor( char* args );
int Stou( char* args );
int Stru( char* args );
int Type( char* args );

/*
   Query and control DTP
*/

/* Transfer mode */
#define TRANSFER_MODE_STREAM     'S'
#define TRANSFER_MODE_BLOCK      'B'
#define TRANSFER_MODE_COMPRESSED 'C'
/* Dataset structure */
#define STRUCTURE_FILE           'F'
#define STRUCTURE_RECORDS        'R'
#define STRUCTURE_PAGES          'P'
/* Representation type */
#define TYPE_ASCII               'A'
#define TYPE_EBCDIC              'E'
#define TYPE_IMAGE               'I'
#define TYPE_LOCAL               'L'
/* Formats */
#define FORMAT_NONPRINT          'N'
#define FORMAT_TELNET            'T'
#define FORMAT_ASA               'A'

char* GetRepresentationType();
char* GetFileStructure();
char* GetTransferMode();
char* GetDataConnType();
char* GetDataConnAddr();

/* Returns RC_TRUE if DTP is sending data through data connection. */
int DataTransferInProgress();
/* Set TransferInProgress mark. */
void StartTransfer();
/* Reset TransferInProgress mark. */
void StopTransfer();
/* Returns RC_TRUE if the data transfer should be aborted. */
int AbortTransfer();
/* 
	Posts the message to DTP to abort the transfer of data. There is some
	dedlay between the message is posted and the transfer is aborted. 
*/
void PostTransferAbort();
/* Cancels abort of data transfer */
void CancelTransferAbort();

/* Type that holds the number of bytes */
typedef unsigned long nb_t;
/* Returns the number of bytes sent so far */
nb_t DataSent();
/* Returns the number of bytes should be sent */
nb_t DataToSend();
/*
  Returns RC_TRUE if the server is started in "passive data connections 
  only" mode (command line option -p). Otherwise returns RC_FALSE.
*/
int PassiveOnlyMode();
/* Sets "passive data connections only" mode. */
void SetPassiveOnlyMode();

/*
   Checksums (MD5)
*/

/* Returns RC_TRUE if the checksums are computed for files to be sent */
int ComputeChecksums();
/* 
	If on is equal to RC_TRUE, the program will compute the checksum of 
	the file to be sent. It will send this sum in the 226 reply through
	the control connection to the peer.
*/
void SetChecksumMode( int on );

/*
   Helper routines for LIST, NLST, MLSD commands 
*/

#include "xbuf.h"
#include "listdir.h"

struct DataConnectionListingParams
{
   XBuf* xb;      /* Buffers output to data connection */
   int   socket;  /* Data connection socket */
   void* params;  /* User-supplied parameters */
};
typedef struct DataConnectionListingParams DCLP;

int DtpLs( char* path, LsCallback cb, void* params );

#endif
