/*
	pi.h

	Declarations for Protocol Interprenter.

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (shesmax@ru.hilti.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: pi.h,v 1.12 2000/08/15 17:31:57 shesmax Exp $ */

#ifndef __GUARD_PI_H
#define __GUARD_PI_H

/*
*	Initialization
*/

/* Initialise the protocol interprenter. Returns 1 on success. */
int  InitPI();
/* Shuts down the interprenter. */
void ShutdownPI();
/* Main loop of the program */
int  RunPI();

/*
*	FTP command handlers
*/

int Abor( char* args );
int Acct( char* args );
int Allo( char* args );
int CdUp( char* args );
int Cwd( char* args );
int Dele( char* args );
int Feat( char* args );
int Help( char* args );
int Mdtm( char* args );
int Mkd( char* args );
int Noop( char* args );
int Opts( char* args );
int Pass( char* args );
int Pwd( char* args );
int Quit( char* args );
int Rein( char* args );
int Rmd( char* args );
int Rnfr( char* args );
int Rnto( char* args );
int Site( char* args );
int Size( char* args );
int Smnt( char* args );
int Stat( char* args );
int Syst( char* args );
int User( char* args );

/*
*	Control connection
*/

/* 
	Returns the name of the peer. The returned string is malloc'ed. It is
	the responsibility of the caller to free it.
*/
char* PeerName();

#include <sys/socket.h>
#include <netinet/in.h>

extern struct sockaddr_in sinPeerCtrl;
extern struct sockaddr_in sinSrvrCtrl;

/*
*	Replies
*/

/*
	Replies are accumulated in the reply buffer. Each time the buffer is full
	its content is sent to the peer.

	Accumulation of reply texts is done with the help of AccumReply... 
	functions. Send... functions write the given text to the reply buffer and
	than send it to the peer.

	For each of these two groups of functions there are three versions:
	
		XXXBuf  	Accepts pointer to the buffer with reply text and the 
					length of the text.
		XXXF  	Allows formatted output.
		XXX  		Is actually a macro that is convinient in use if you know the 
					length of the text at compile time. This is true in most places
					of the code.

	Please note that formated output creates a temporary buffer to make
	a reply text and then sends this text to the reply buffer. Use
	formatted output only in the cases where the length of the resulting
	text is under your control. 
	

	Return codes
 	
		RC_OK 				Text is saved in the reply buffer.
		RC_TIMEDOUT			The program attempted to send whole or part of reply 
								text to peer. It was failed to do so, because peer 
								did'nt respond. The content of the buffer is undefined
								from now on. 
		RC_ERROR				The program attempted to send whole or part of reply 
								text to peer. It was failed to do so, because of some
								network or OS error. The content of the buffer is 
								undefined from now on. 
*/
#ifndef _SIZE_T
#	include <stdarg.h>	/* for size_t */
#endif

int AccumReplyBuf( const char* buf, size_t nBytes );
int AccumReplyF( const char* fmt, ... );
#define tic inline void bug(void) { 0 = 1; }
#define AccumReply( text ) \
   ({ typeof(text) x __attribute__((__unused__)) = {}; \
         AccumReplyBuf( text, sizeof( text ) - 1 ); })

int SendReplyBuf( const char* text, size_t nBytes );
int SendReplyF( const char* text, ... );
#define SendReply( text ) \
   ({ typeof(text) x __attribute__((__unused__)) = {}; \
         SendReplyBuf( text, sizeof( text ) - 1 ); })

/*
	Sends all accumulated replies (if any) to the remote host.
	
	Returns

		RC_OK				Reply was sent to the peer.
		RC_ERROR			An error occured during the transmition of reply.
		RC_TIMEDOUT		Timeout expired during the transmition of reply.
*/
int SendReplies();

/* Accumulates the reply who consists of nCode and system error string */
void PError( int nCode );

/*
*	Miscelaneous
*/

/* 
	Sends the content of the file .message in the current directory through
	the control connection. 
	
	Parameters
	
	code	reply code 
	sPfx	prefix line (have to be sent in any case)
*/
int DirInfo( int code, const char* sPfx );

/* 
	Constructs the line which prefixes the directory entry's name in
	output from ls -l. 

	Parameters

		st				file attributes
		sPrefix		buffer for prefix
		nBytes		size of the buffer
*/
struct stat;
char* LsLongPrefix( const struct stat* st, char* sPrefix, int nBytes );

#endif
