/*
   sockets.h

   Wrappers for socket calls

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (shesmax@ru.hilti.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: sockets.h,v 1.8 2000/08/15 17:31:57 shesmax Exp $ */

#ifndef __GUARD_SOCKETS_H
#define __GUARD_SOCKETS_H

#define INVALID_SOCKET  -1

/*
*  Socket operations
*/

/* 
   Writes the given data to the socket nSocket.
   
   Returns:
   RC_OK          no errors
   RC_ERROR       some error occured during write (see errno)
   RC_ABORTED     data transfer aborted by the client (this can happen
                  for data connections only)
   RC_TIMEDOUT    operation timed out. 
*/
extern int WriteSocket( int nSocket, const void* pBuf, int nCount );

/* 
   Reads the data from the socket nSocket into the buffer, but not more
   then nCount bytes.
   
   Returns:
   number of bytes read so far if there were no errors.
   RC_ERROR       some error occured during read (see errno)
   RC_ABORTED     data transfer aborted by the client (this can happen
                  for data connections only)
   RC_TIMEDOUT    operation timed out. 
*/
extern int ReadSocket( int nSocket, void* pBuf, int nCount );

/* 
   Connects the socket nSocket to the remote host whose address is in addr.
   
   Returns:
   RC_OK          no errors
   RC_ERROR       OS error
   RC_TIMEDOUT    operation timed out 
*/
struct sockaddr_in;
extern int ConnectSocket( int nSocket, struct sockaddr_in* addr );

/* 
   Accepts the connection request on the socket nSocket.
   Returns either socket handle or INVALID_SOCKET. 
*/
extern int AcceptSocket( int nSocket, struct sockaddr_in* addr );

/*
*  Timeout values
*/

#ifndef _SYS_TYPES_H
#  include <sys/types.h>
#endif

/* Timeout configuration through recompilation only */
#define DEFAULT_TIMEOUT    300         /* 5 minutes (RFC 1123) */

/* Returns the timeout value for socket operation */
unsigned long GetSocketTimeout();

/* 
   Sets a new timeout value. This should be less then DEFAULT_TIMEOUT.
   Returns RC_TRUE(1) if a new value is set, otherwise returns RC_FALSE(0) 
*/
int SetSocketTimeout( unsigned long nNewTimeout );

#endif
